const fs = require('fs');
const path = require('path');
const xml2js = require('xml2js');
const XLSX = require('xlsx');
const { Pool } = require('pg');

// Get filenames from command-line arguments
const workbookFilename = process.argv[2];
const xmlFilename = process.argv[3];
const outputFilename = process.argv[4] || 'output.json';

if (!workbookFilename || !xmlFilename) {
  console.log('Please provide the filenames for the workbook and XML files as command-line arguments.');
  console.log('Usage: node import_prpay.js <workbook-filename> <xml-filename> [output-filename]');
  process.exit(1);
}

// PostgreSQL connection configuration
const pool = new Pool({
  user: 'root',
  host: 'localhost',
  database: 'myform',
  password: 'password',
  port: '5432',
});

// Read the XML file
const xmlString = fs.readFileSync(xmlFilename, 'utf-8');

// Parse the XML into a JavaScript object
xml2js.parseString(xmlString, async (err, result) => {
  if (err) {
    console.error('Error parsing XML:', err);
    return;
  }

  // Read the values dynamically from the XML tags under <employee> tag
  const employee = result.root.employee[0];
  const employeeidTag = employee.employeeid[0];
  const incTags = employee.formulaid[0].inc;
  const dctTags = employee.formulaid[0].dct;

  // Combine INC and DCT tags
  const formulaidTags = [...incTags, ...dctTags];

  // Read the Excel file
  const workbook = XLSX.readFile(workbookFilename);
  const worksheet = workbook.Sheets[workbook.SheetNames[0]];
  const data = XLSX.utils.sheet_to_json(worksheet, { header: 1 });

  // Find the index of the employeeid column
  const employeeidColumnIndex = data[0].indexOf('employeeid');

  const jsonData = {
    employee: [],
  };

  let skippedRecords = 0;
  let importedRecords = 0;

  const startTime = new Date();

  for (let i = 1; i < data.length; i++) {
    const row = data[i];
    const mappedEmployee = {};

    // Map the employeeid value from the Excel column
    const employeeid = row[employeeidColumnIndex];
    mappedEmployee.employeeid = replacePlaceholders(employeeidTag, employeeid);

    mappedEmployee.formulaid = formulaidTags.reduce((result, tag) => {
      const { column, value } = tag.$;
      const columnIndex = data[0].indexOf(column);

      if (columnIndex !== -1 && row[columnIndex] !== undefined) {
        const tagType = column.startsWith('INC') ? 'INC' : 'DCT';
        result.push({
          tagType,
          formulaid: replacePlaceholders(column, row[columnIndex]),
          value: replacePlaceholders(value, row[columnIndex]),
        });
      }

      return result;
    }, []);

    jsonData.employee.push(mappedEmployee);

    // Calculate progress
    const progress = (i / (data.length - 1)) * 100;
    console.log(`Progress: ${progress.toFixed(2)}%`);
  }

  try {
    // Connect to the PostgreSQL database
    const client = await pool.connect();

    // Insert the data into the PostgreSQL database
    const tableName = 'tpytran1'; // Specify the table name
    for (const employee of jsonData.employee) {
      const { employeeid, formulaid } = employee;

      for (const formula of formulaid) {
        const { tagType, formulaid: formulaId, value } = formula;

        const query = `INSERT INTO ${tableName} (employeeid, formulaid, cur_amount, companyid) VALUES ($1, $2, $3,'1')`;
        const values = [employeeid, formulaId, value];

        try {
          await client.query(query, values);
          importedRecords++;
        } catch (err) {
          console.error('Error inserting data:', err);
          skippedRecords++;
          // Add retry logic or handle the error accordingly
        }
      }
    }

    client.release(); // Release the client back to the pool

    const endTime = new Date();
    const executionTime = (endTime - startTime) / 1000;

    // Convert JSON to string
    const jsonString = JSON.stringify(jsonData, null, 2);

    // Write JSON to file
    fs.writeFileSync(outputFilename, jsonString);

    console.log(`Data processed successfully.`);
    console.log(`Total records: ${data.length - 1}`);
    console.log(`Skipped records: ${skippedRecords}`);
    console.log(`Imported records: ${importedRecords}`);
    console.log(`Execution time: ${executionTime.toFixed(2)} seconds`);
    console.log(`Conversion completed. Output JSON file created: ${outputFilename}`);
  } catch (err) {
    console.error('Error connecting to the database:', err);
  }
});

// Function to replace placeholders with actual values
function replacePlaceholders(str, value) {
  return str.replace(/{(\w+)}/g, value);
}
